//////////////////////////////////////////////////////////////////////
//                                                                  //
//  CluelessPlus - an noai-AI                                       //
//                                                                  //
//////////////////////////////////////////////////////////////////////
// 
// Author: Zuu (Leif Linse), user Zuu @ tt-forums.net
// Purpose: To play around with the noai framework.
//               - Not to make the next big thing.
// Copyright: Leif Linse - 2008, 2009
// License: GNU GPL - version 2

class Helper {
	company_name = null; // a copy of state_company_name which does not include the state-prefix that is included if one ask company.GetCompanyName();

	constructor() {
		company_name = "";
	}

	static function IsStraight(tile1, tile2);
	static function Distance(a, b);
	static function TownDistance(town1, town2);
	static function StepFunction(t);
	static function ArrayFind(array, toFind);
	static function TileLocationString(tile);
	static function SetSign(tile, message);
	static function BreakPoint(sign_tile);
	static function HasSign(text);
	static function ClearAllSigns();
	static function IsTownInConnectionList(connection_list, check_town);
	static function Debug(text);

	static function ItemValuator(item);
}

function Helper::TownDistance(town1, town2)
{
	return AIMap.DistanceManhattan(AITown.GetLocation(town1), AITown.GetLocation(town2));
}

function Helper::Distance(a, b)
{
	return AIMap.DistanceManhattan(a, b);
}

function Helper::StepFunction(t)
{
	if(t>=0)
		return 1;

	return 0;
}

function Helper::TileLocationString(tile)
{
	return "(" + AIMap.GetTileX(tile) + ", " + AIMap.GetTileY(tile) + ")";
}

function Helper::SetSign(tile, message)
{
	if(AIController.GetSetting("debug_signs") != 1)
		return;

	local found = false;
	for(local i = 0; i < AISign.GetMaxSignID(); ++i)
	{
		if(AISign.IsValidSign(i) && AISign.GetLocation(i) == tile)
		{
			if(found)
				AISign.RemoveSign(i);
			else
			{
				if(message == "")
					AISign.RemoveSign(i);
				else
					AISign.SetName(i, message);
				found = true;
			}
		}
	}

	if(!found)
		AISign.BuildSign(tile, message);
}

// Places a sign on tile sign_tile and waits until the sign gets removed
function Helper::BreakPoint(sign_tile)
{
	if(AIController.GetSetting("debug_signs") != 1)
		return;

	if(Helper.HasSign("no_break"))
		return;

	AILog.Warning("Break point reached. -> Remove the \"break\" sign to continue.");
	local sign = AISign.BuildSign(sign_tile, "break");
	while(AISign.IsValidSign(sign)) { AIController.Sleep(1); }
}

function Helper::HasSign(text)
{
	for(local i = 0; i < AISign.GetMaxSignID(); ++i)
	{
		if(AISign.IsValidSign(i) && AISign.GetName(i) == text)
		{
			return true;
		}
	}
	return false;
}
function Helper::ClearAllSigns()
{
	local i = 0;
	for(i = 0; i < AISign.GetMaxSignID(); ++i)
	{
		if(i != null && AISign.IsValidSign(i))
		{
			AISign.RemoveSign(i);
		}
	}
}

function Helper::IsStraight(tile1, tile2)
{
	if(AIMap.GetTileX(tile1) == AIMap.GetTileX(tile2) ||
			AIMap.GetTileY(tile1) == AIMap.GetTileY(tile2))
	{
		return true;
	}
	return false;
}

function Helper::IsTownInConnectionList(connection_list, check_town)
{
	foreach(val in connection_list)
	{
		foreach(town in val.town)
		{
			if(town == check_town)
			{
				return true;
			}
		}
	}
	return false;
}

function Helper::Debug(text)
{
	//print(company_name + ": " + text);
	AILog.Info(text);
}

// RETURN null if not found, else the key to the found value.
function Helper::ArrayFind(array, toFind)
{
	
	foreach(key, val in array)
	{
		if(val == toFind)
		{
			return key;
		}
	}
	return null;
}

function Helper::GetPAXCargo()
{
	local cargo_list = AICargoList();
	cargo_list.Valuate(AICargo.HasCargoClass, AICargo.CC_PASSENGERS);
	cargo_list.KeepValue(1);
	cargo_list.Valuate(AICargo.GetTownEffect);
	cargo_list.KeepValue(AICargo.TE_PASSENGERS);

	if(!AICargo.IsValidCargo(cargo_list.Begin()))
	{
		AILog.Error("PAX Cargo do not exist");
	}

	//AILog.Info("PAX Cargo is: " + AICargo.GetCargoLabel(cargo_list.Begin()));

	return cargo_list.Begin();
}

function Helper::ItemValuator(item)
{
	return item;
}
