//////////////////////////////////////////////////////////////////////
//                                                                  //
//  CluelessPlus - an noai-AI                                       //
//                                                                  //
//////////////////////////////////////////////////////////////////////
// 
// Author: Zuu (Leif Linse), user Zuu @ tt-forums.net
// Purpose: To play around with the noai framework.
//               - Not to make the next big thing.
// Copyright: Leif Linse - 2008, 2009
// License: GNU GPL - version 2


import("pathfinder.road", "RPF", 3);
require("helper.nut");

class RoadBuilder {
	helper = null;

	constructor(helper) {
		this.helper = helper;
	}
	

	static function ConnectTiles(tile1, tile2, call_count);

}


function RoadBuilder::ConnectTiles(tile1, tile2, call_count) {
	local pathfinder = RPF();

	pathfinder.InitializePath([tile1], [tile2]);

	local path = false;
	while (path == false) {
		path = pathfinder.FindPath(100);
		AIController.Sleep(1);
	}

	if(path == null) {
		AILog.Info("Failed to find path");
		return false;
	}
	AILog.Info("Path found, now start building!");

	//AISign.BuildSign(path.GetTile(), "Start building");

	while (path != null) {
		local par = path.GetParent();
		if (par != null) {
			local last_node = path.GetTile();
			if (AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) == 1 ) {
				if (AIRoad.AreRoadTilesConnected(path.GetTile(), par.GetTile())) {
					// there is allready a road here, don't do anything
				} else {

					/* Look for longest straight road and build it as one build command */
					local straight_begin = path;
					local straight_end = par;
					local prev = straight_end.GetParent();
					while(prev != null && 
							helper.IsStraight(straight_begin.GetTile(), prev.GetTile()) &&
							AIMap.DistanceManhattan(straight_end.GetTile(), prev.GetTile()) == 1)
					{
						straight_end = prev;
						prev = straight_end.GetParent();
					}

					/* update the looping vars. (path is set to par in the end of the main loop) */
					par = straight_end;

					//AISign.BuildSign(path.GetTile(), "path");
					//AISign.BuildSign(par.GetTile(), "par");

					//if (!AIRoad.BuildRoad(path.GetTile(), par.GetTile())) {
					if (!AIRoad.BuildRoad(straight_begin.GetTile(), straight_end.GetTile())) {
						/* An error occured while building a piece of road. TODO: handle it. 
						 * Note that is can also be the case that the road was already build. */

						// Try PF again
						if (call_count > 1 || !RoadBuilder.ConnectTiles(par.GetTile(), path.GetTile(), call_count+1))
							return false;
					}
				}
			} else {
				/* Build a bridge or tunnel. */
				if (!AIBridge.IsBridgeTile(path.GetTile()) && !AITunnel.IsTunnelTile(path.GetTile())) {
					/* If it was a road tile, demolish it first. Do this to work around expended roadbits. */
					if (AIRoad.IsRoadTile(path.GetTile())) AITile.DemolishTile(path.GetTile());
					if (AITunnel.GetOtherTunnelEnd(path.GetTile()) == par.GetTile()) {
						if (!AITunnel.BuildTunnel(AIVehicle.VT_ROAD, path.GetTile())) {
							/* An error occured while building a tunnel. TODO: handle it. */
							helper.Debug("Build tunnel error: " + AIError.GetLastErrorString());
						}
					} else {
						local bridge_list = AIBridgeList_Length(AIMap.DistanceManhattan(path.GetTile(), par.GetTile()) +1);
						bridge_list.Valuate(AIBridge.GetMaxSpeed);
						if (!AIBridge.BuildBridge(AIVehicle.VT_ROAD, bridge_list.Begin(), path.GetTile(), par.GetTile())) {
							/* An error occured while building a bridge. TODO: handle it. */
							helper.Debug("Build bridge error: " + AIError.GetLastErrorString());
						}
					}
				}
			}
		}
		path = par;
	}

	//AISign.BuildSign(tile1, "Done");

	return true;
}
