//////////////////////////////////////////////////////////////////////
//                                                                  //
//  CluelessPlus - an noai-AI                                       //
//                                                                  //
//////////////////////////////////////////////////////////////////////
// 
// Author: Zuu (Leif Linse), user Zuu @ tt-forums.net
// Purpose: To play around with the noai framework.
//               - Not to make the next big thing.
// Copyright: Leif Linse - 2009
// License: GNU GPL - version 2


//////////////////////////////////////////////////////
//                                                  //
//   CLASS: StationStatistics                       //
//                                                  //
//////////////////////////////////////////////////////

class StationStatistics {
	station_id = null;
	pax_waiting = null;
	rating = null;

	usage = null;

	bus_usage = null;
	truck_usage = null;
	aircraft_usage = null;
	train_usage = null;

	constructor(station_id)
	{
		this.station_id = station_id;
		pax_waiting = -1;
		rating = -1;
		usage = { bus =    {station_type = AIStation.STATION_BUS_STOP,   mode = AIVehicle.VT_ROAD, percent_usage = -1}, 
				truck =    {station_type = AIStation.STATION_TRUCK_STOP, mode = AIVehicle.VT_ROAD, percent_usage = -1},
				aircraft = {station_type = AIStation.STATION_AIRPORT,    mode = AIVehicle.VT_AIR,  percent_usage = -1},
				train =    {station_type = AIStation.STATION_TRAIN,      mode = AIVehicle.VT_RAIL, percent_usage = -1} };
	}

	static function VehicleIsWithinTileList(vehicle_id, tile_list);
};

function StationStatistics::VehicleIsWithinTileList(vehicle_id, tile_list)
{
	return tile_list.HasItem(AIVehicle.GetLocation(vehicle_id));
}

function StationStatistics::GetNumAircraftsInAirportQueue(station_id)
{
	// aircrafts
	local station_vehicle_list = AIVehicleList_Station(station_id);
	station_vehicle_list.Valuate(AIVehicle.GetVehicleType);
	station_vehicle_list.KeepValue(AIVehicle.VT_AIR);

	// get airport tile
	local station_tile_list = AITileList_StationType(station_id, AIStation.STATION_AIRPORT);
	station_tile_list.Valuate(Helper.DummyValuator);
	station_tile_list.Sort(AIAbstractList.SORT_BY_VALUE, true); // smallest first

	local airport_tile = station_tile_list.Begin();
	//local airport_tile = Builder.GetAirportTile(station_id);

	// get airport holding rectangle
	local holding_rect = AITileList();	

	switch(AIAirport.GetAirportType(airport_tile))
	{
		case AIAirport.AT_SMALL:
			holding_rect.AddRectangle(Helper.GetTileRelative(airport_tile, -2, 0), Helper.GetTileRelative(airport_tile, 17, 12));
			holding_rect.RemoveRectangle(Helper.GetTileRelative(airport_tile, 1, 1), Helper.GetTileRelative(airport_tile, 3, 2)); // remove non-holding airport tiles
			break;
		
		case AIAirport.AT_COMMUTER:
			holding_rect.AddRectangle(Helper.GetTileRelative(airport_tile, -2, 0), Helper.GetTileRelative(airport_tile, 14, 9));
			holding_rect.RemoveRectangle(Helper.GetTileRelative(airport_tile, 1, 1), Helper.GetTileRelative(airport_tile, 4, 3)); // remove non-holding airport tiles
			break;

		case AIAirport.AT_LARGE:
			holding_rect.AddRectangle(Helper.GetTileRelative(airport_tile, 9, 0), Helper.GetTileRelative(airport_tile, 17, 5));
			break;

		case AIAirport.AT_METROPOLITAN:
			holding_rect.AddRectangle(Helper.GetTileRelative(airport_tile, -2, 0), Helper.GetTileRelative(airport_tile, 17, 12));
			holding_rect.RemoveRectangle(Helper.GetTileRelative(airport_tile, 1, 1), Helper.GetTileRelative(airport_tile, 5, 5)); // remove non-holding airport tiles
			break;

		case AIAirport.AT_INTERNATIONAL:
			holding_rect.AddRectangle(Helper.GetTileRelative(airport_tile, -2, 0), Helper.GetTileRelative(airport_tile, 19, 13));
			holding_rect.RemoveRectangle(Helper.GetTileRelative(airport_tile, 1, 1), Helper.GetTileRelative(airport_tile, 6, 6)); // remove non-holding airport tiles
			break;

		case AIAirport.AT_INTERCON:
			holding_rect.AddRectangle(Helper.GetTileRelative(airport_tile, -13, -11), Helper.GetTileRelative(airport_tile, 19, 21));
			holding_rect.RemoveRectangle(Helper.GetTileRelative(airport_tile, 0, 0), Helper.GetTileRelative(airport_tile, 8, 10)); // remove non-holding airport tiles
			break;

		default:
			// Unknown airport type -> crash the AI
			KABOOOOOM_unknown_airport_type();
	}

	station_vehicle_list.Valuate(VehicleIsWithinTileList, holding_rect);
	station_vehicle_list.KeepValue(1);

	// Only keep vehicles that are on the way to the airport
	station_vehicle_list.Valuate(Order.GetCurrentOrderDestination);
	station_vehicle_list.KeepValue(AIStation.GetLocation(station_id));

	// remove vehicles that are in a depot (the right depot of international would else give false positive
	station_vehicle_list.Valuate(AIVehicle.IsInDepot);
	station_vehicle_list.KeepValue(0);

	return station_vehicle_list.Count();
}

function StationStatistics::GetNumNonStopedAircraftsInAirportDepot(station_id)
{
	local hangar_vehicles = Builder.GetAircraftsInHangar(station_id);

	hangar_vehicles.Valuate(AIVehicle.IsStoppedInDepot);
	hangar_vehicles.KeepValue(0);

	return hangar_vehicles.Count();
}

function StationStatistics::ReadStatisticsData()
{
	//AILog.Info("ReadStatisticsData for station " + AIStation.GetName(this.station_id));

	local alpha = 4; // weight of old value (new value will have weight 1)

	local currently_waiting = AIStation.GetCargoWaiting(this.station_id, Helper.GetPAXCargo());
	if(this.pax_waiting == -1)
		this.pax_waiting = currently_waiting;
	else
		this.pax_waiting = (this.pax_waiting * alpha + currently_waiting) / (alpha + 1);

	//AILog.Info("pax_waiting = " + this.pax_waiting);

	local current_rating = AIStation.GetCargoRating(this.station_id, Helper.GetPAXCargo());
	if(this.rating == -1)
		this.rating = current_rating;
	else
		this.rating = (this.rating * alpha + current_rating) / (alpha + 1);

	//AILog.Info("rating = " + this.rating);

	// Get an estimate on the vehicle load of the station, by counting the number of vehicles on station tiles
	foreach(_, transp_mode_usage in this.usage)
	{
		if(AIStation.HasStationType(this.station_id, transp_mode_usage.station_type))
		{

			if(transp_mode_usage.station_type == AIStation.STATION_AIRPORT)
			{
				local hangar_num = GetNumNonStopedAircraftsInAirportDepot(this.station_id);
				local holding_num = GetNumAircraftsInAirportQueue(this.station_id);
				local new_percent_usage = (holding_num + hangar_num)* 100;
				if(transp_mode_usage.percent_usage == -1)
					transp_mode_usage.percent_usage = new_percent_usage;
				else
					transp_mode_usage.percent_usage = (transp_mode_usage.percent_usage * alpha + new_percent_usage) / (alpha + 1);

				Helper.SetSign(AIStation.GetLocation(station_id) + 2, "hold: " + holding_num + " han: " + hangar_num);
				Helper.SetSign(AIStation.GetLocation(station_id) + 3, "queue: " + (new_percent_usage / 100));
			}
			else
			{
				local station_tile_list = AITileList_StationType(this.station_id, transp_mode_usage.station_type);
				local station_vehicle_list = AIVehicleList_Station(this.station_id);
				station_vehicle_list.Valuate(AIVehicle.GetVehicleType);
				station_vehicle_list.KeepValue(transp_mode_usage.mode);

				local num_vehicles_on_station = 0;

				for(local vehicle = station_vehicle_list.Begin(); station_vehicle_list.HasNext(); vehicle = station_vehicle_list.Next())
				{
					for(local tile = station_tile_list.Begin(); station_tile_list.HasNext(); tile = station_tile_list.Next())
					{
						if(AIVehicle.GetLocation(vehicle) == tile)
						{
							num_vehicles_on_station++;
						}
					}
				}

				local new_percent_usage = num_vehicles_on_station * 100 / station_tile_list.Count();

				if(transp_mode_usage.percent_usage == -1)
					transp_mode_usage.percent_usage = new_percent_usage;
				else
					transp_mode_usage.percent_usage = (transp_mode_usage.percent_usage * alpha + new_percent_usage) / (alpha + 1);
			}
		}
		else
		{
			transp_mode_usage.percent_usage = -1;
		}
	}

	//AILog.Info("bus usage = " + this.usage.bus.percent_usage);

	Helper.SetSign(AIStation.GetLocation(station_id), "w" + pax_waiting + " r" + rating + " u" + usage.bus.percent_usage + " a" + usage.aircraft.percent_usage);
}
